/*

*/

import renderer.scene.*;
import renderer.models.SierpinskiTriangle;
import renderer.models.ModelShading;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**

*/
public class SierpinskiMovie
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Create a Position object that will hold the robot arm.
      Position top_p = new Position();

      // Add the Position object to the scene.
      scene.addPosition(top_p);

      // Push the position away from where the camera is.
      top_p.matrix.mult( Matrix.translate(0, 0, -1) );
      top_p.matrix.mult( Matrix.rotateZ(90) );

      // Create the Model object.
      Model sierpinskiTriangle = new SierpinskiTriangle(8);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(0), Color.blue);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(1), Color.red);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(2), Color.magenta);

      // Add the model to the position.
      top_p.model = sierpinskiTriangle;

      // Create a framebuffer to render our scene into.
      int vp_width  = 1024;
      int vp_height = 1024;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);

      for (int k = 0; k < 720; k++)
      {
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_SierpinskiMovie_Frame%03d.ppm", k));
       //fb.dumpFB2File(String.format("PNG_SierpinskiMovie_Frame%03d.png", k), "png");

         updateNestedMatrices(sierpinskiTriangle, true);
      }
   }//main()


   private static void updateNestedMatrices(Model model, boolean both)
   {
      if (! model.nestedModels.isEmpty())
      {
         if (both) {
            model.nestedModels.get(1).nestedMatrix = model.nestedModels.get(1).nestedMatrix.times(Matrix.rotateZ( 0.5));
            model.nestedModels.get(2).nestedMatrix = model.nestedModels.get(2).nestedMatrix.times(Matrix.rotateZ(-0.5));
         } else {
            model.nestedModels.get(0).nestedMatrix = model.nestedModels.get(0).nestedMatrix.times(Matrix.rotateZ( 0.5));
         }
         for (Model m : model.nestedModels) { updateNestedMatrices(m, both); }
      }
   }
}
