/*

*/

import renderer.scene.*;
import renderer.models.SierpinskiTriangle;
import renderer.models.ModelShading;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**

*/
public class SierpinskiMovie
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Create a Position object that will hold the robot arm.
      Position top_p = new Position();

      // Add the Position object to the scene.
      scene.addPosition(top_p);

      // Push the position away from where the camera is.
      top_p.matrix.mult( Matrix.translate(0, 0, -1) );
      top_p.matrix.mult( Matrix.rotateZ(90) );

      // Create a framebuffer to render our scene into.
      int vp_width  = 800;
      int vp_height = 800;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);

      final double angle = 1.0;

      // Create the Model object.
      Model sierpinskiTriangle = new SierpinskiTriangle(8);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(0), Color.blue);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(1), Color.red);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(2), Color.magenta);
      top_p.model = sierpinskiTriangle;
      for (int k = 0; k < 120; k++)
      {
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_SierpinskiMovie_Frame%04d.ppm", k));
       //fb.dumpFB2File(String.format("PNG_SierpinskiMovie_Frame%04d.png", k), "png");

         updateNestedMatrices_1(sierpinskiTriangle, angle);
      }

      // Create a new Model object.
      sierpinskiTriangle = new SierpinskiTriangle(8);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(0), Color.blue);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(1), Color.red);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(2), Color.magenta);
      top_p.model = sierpinskiTriangle;
      for (int k = 0; k < 240; k++)
      {
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_SierpinskiMovie_Frame%04d.ppm", 120+k));
       //fb.dumpFB2File(String.format("PNG_SierpinskiMovie_Frame%04d.png", 120+k), "png");

         updateNestedMatrices_2(sierpinskiTriangle, angle);
      }

      // Create a new Model object.
      sierpinskiTriangle = new SierpinskiTriangle(8);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(0), Color.blue);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(1), Color.red);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(2), Color.magenta);
      top_p.model = sierpinskiTriangle;
      for (int k = 0; k < 240; k++)
      {
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_SierpinskiMovie_Frame%04d.ppm", 360+k));
       //fb.dumpFB2File(String.format("PNG_SierpinskiMovie_Frame%04d.png", 360+k), "png");

         updateNestedMatrices_4(sierpinskiTriangle, angle);
      }

      // Create a new Model object.
      sierpinskiTriangle = new SierpinskiTriangle(8);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(0), Color.blue);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(1), Color.red);
      ModelShading.setColor(sierpinskiTriangle.nestedModels.get(2), Color.magenta);
      top_p.model = sierpinskiTriangle;
      for (int k = 0; k < 360; k++)
      {
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_SierpinskiMovie_Frame%04d.ppm", 600+k));
       //fb.dumpFB2File(String.format("PNG_SierpinskiMovie_Frame%04d.png", 600+k), "png");

         updateNestedMatrices_3(sierpinskiTriangle, angle);
      }

   }//main()


   private static void updateNestedMatrices_1(Model model, double angle)
   {
      if (! model.nestedModels.isEmpty())
      {
         model.nestedModels.get(0).nestedMatrix = model.nestedModels.get(0).nestedMatrix.times(Matrix.rotateZ(angle));
         model.nestedModels.get(1).nestedMatrix = model.nestedModels.get(1).nestedMatrix.times(Matrix.rotateZ(angle));
         model.nestedModels.get(2).nestedMatrix = model.nestedModels.get(2).nestedMatrix.times(Matrix.rotateZ(angle));

         for (Model m : model.nestedModels)
         {
            updateNestedMatrices_1(m, angle);
         }
      }
   }

   private static void updateNestedMatrices_2(Model model, double angle)
   {
      if (! model.nestedModels.isEmpty())
      {
         model.nestedModels.get(0).nestedMatrix = model.nestedModels.get(0).nestedMatrix.times(Matrix.rotateZ(angle));

         for (Model m : model.nestedModels)
         {
            updateNestedMatrices_2(m, angle);
         }
      }
   }

   private static void updateNestedMatrices_3(Model model, double angle)
   {
      if (! model.nestedModels.isEmpty())
      {
         model.nestedModels.get(1).nestedMatrix = model.nestedModels.get(1).nestedMatrix.times(Matrix.rotateZ( angle));
         model.nestedModels.get(2).nestedMatrix = model.nestedModels.get(2).nestedMatrix.times(Matrix.rotateZ(-angle));

         for (Model m : model.nestedModels)
         {
            updateNestedMatrices_3(m, angle);
         }
      }
   }

   private static void updateNestedMatrices_4(Model model, double angle)
   {
      if (! model.nestedModels.isEmpty())
      {
         model.nestedModels.get(1).nestedMatrix = model.nestedModels.get(1).nestedMatrix.times(Matrix.rotateZ(-angle));
         model.nestedModels.get(2).nestedMatrix = model.nestedModels.get(2).nestedMatrix.times(Matrix.rotateZ(-angle));

         for (Model m : model.nestedModels)
         {
            updateNestedMatrices_4(m, angle);
         }
      }
   }
}
