/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.awt.Color;
import java.awt.event.*;

/**
   Draw an interactive robot arm with one shoulder,
   two elbow, two wrist, and two finger joints.
*/
public class RobotArms1
{
   private static double xTranslation = 0.0;
   private static double yTranslation = 0.0;

   private static double shoulderRotation =  0.0;
   private static double   elbowRotation1 =  15.0;
   private static double   elbowRotation2 = -15.0;
   private static double   wristRotation1 =  0.0;
   private static double   wristRotation2 =  0.0;
   private static double  fingerRotation1 =  0.0;
   private static double  fingerRotation2 =  0.0;

   private static double shoulderLength = 0.4;
   private static double    elbowLength = 0.3;
   private static double    wristLength = 0.2;
   private static double   fingerLength = 0.1;

   private static void print_help_message()
   {
      //System.out.println("Use the 'd' key to toggle debugging information on and off.");
      //System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the s/S keys to rotate the arm at the shoulder.");
      System.out.println("Use the e/E keys to rotate the arm at elbow 1.");
      System.out.println("Use the w/W keys to rotate the arm at wrist 1.");
      System.out.println("Use the f/F keys to rotate the arm at finger 1.");
      System.out.println("Use the d/D keys to rotate the arm at elbow 2.");
      System.out.println("Use the a/A keys to rotate the arm at wrist 2.");
      System.out.println("Use the c/C keys to rotate the arm at finger 2.");
      System.out.println("Use the ^s/^S keys to extend the length of the arm at the shoulder.");
      System.out.println("Use the ^e/^E keys to extend the length of the arm at the elbow.");
      System.out.println("Use the ^w/^W keys to extend the length of the arm at the wrist.");
      System.out.println("Use the ^f/^F keys to extend the length of the arm at the finger.");
      System.out.println("Use the x/X keys to translate the arm along the x-axis.");
      System.out.println("Use the y/Y keys to translate the arm along the y-axis.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }


   public static void main(String[] args)
   {
      print_help_message();

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Create a Position object that will hold the robot arm.
      Position arm_p = new Position();

      // Add the Position object to the scene.
      scene.addPosition(arm_p);

      /*
         Be sure to draw a picture of the tree that this code creates.
      */
      Vertex v0 = new Vertex(0, 0, 0);
      Vertex v1 = new Vertex(1, 0, 0);
      Model arm = new Model();
      arm.addVertex(v0, v1);
      arm.addLineSegment(new LineSegment(0, 1));
      // Add the arm Model to the Scene's Position.
      arm_p.model = arm;

      // two elbows
      Model elbow1 = new Model();
      elbow1.addVertex(v0, v1);
      elbow1.addLineSegment(new LineSegment(0, 1));
      arm.nestedModels.add( elbow1 );

      Model elbow2 = new Model();
      elbow2.addVertex(v0, v1);
      elbow2.addLineSegment(new LineSegment(0, 1));
      arm.nestedModels.add( elbow2 );

      // two wrists
      Model wrist1 = new Model();
      wrist1.addVertex(v0, v1);
      wrist1.addLineSegment(new LineSegment(0, 1));
      elbow1.nestedModels.add( wrist1 );

      Model wrist2 = new Model();
      wrist2.addVertex(v0, v1);
      wrist2.addLineSegment(new LineSegment(0, 1));
      elbow2.nestedModels.add( wrist2 );

      // two fingers
      Model finger1 = new Model();
      finger1.addVertex(v0, v1);
      finger1.addLineSegment(new LineSegment(0, 1));
      wrist1.nestedModels.add( finger1 );

      Model finger2 = new Model();
      finger2.addVertex(v0, v1);
      finger2.addLineSegment(new LineSegment(0, 1));
      wrist2.nestedModels.add( finger2 );

      arm.setColor( Color.blue );

      // Push the Position away from where the camera is.
      arm_p.matrix2Identity();
      arm_p.translate(xTranslation, yTranslation, -1);

      // Initialize the nested matrices for the sub models.
      arm.nestedMatrix     = Matrix.rotateZ(shoulderRotation)
                            .times(Matrix.scale(shoulderLength, shoulderLength, 1));
      elbow1.nestedMatrix  = Matrix.translate(1, 0, 0)
                            .times(Matrix.rotateZ(elbowRotation1))
                            .times(Matrix.scale(elbowLength/shoulderLength, elbowLength/shoulderLength, 1));
      elbow2.nestedMatrix  = Matrix.translate(1, 0, 0)
                            .times(Matrix.rotateZ(elbowRotation2))
                            .times(Matrix.scale(elbowLength/shoulderLength, elbowLength/shoulderLength, 1));
      wrist1.nestedMatrix  = Matrix.translate(1, 0, 0)
                            .times(Matrix.rotateZ(wristRotation1))
                            .times(Matrix.scale(wristLength/elbowLength, wristLength/elbowLength, 1));
      wrist2.nestedMatrix  = Matrix.translate(1, 0, 0)
                            .times(Matrix.rotateZ(wristRotation2))
                            .times(Matrix.scale(wristLength/elbowLength, wristLength/elbowLength, 1));
      finger1.nestedMatrix = Matrix.translate(1, 0, 0)
                            .times(Matrix.rotateZ(fingerRotation1))
                            .times(Matrix.scale(fingerLength/wristLength, fingerLength/wristLength, 1));
      finger2.nestedMatrix = Matrix.translate(1, 0, 0)
                            .times(Matrix.rotateZ(fingerRotation2))
                            .times(Matrix.scale(fingerLength/wristLength, fingerLength/wristLength, 1));


      // Define initial dimensions for a FrameBuffer.
      int width  = 512;
      int height = 512;

      // Create an InteractiveFrame containing a FrameBuffer
      // with the given dimensions.
      @SuppressWarnings("serial")
      InteractiveFrame app = new InteractiveFrame("Renderer 9", width, height)
      {
         @Override public void keyPressed(KeyEvent e)
         {
          //System.out.println( e );

            if (e.isControlDown() && e.isShiftDown() && e.getKeyCode() == 83) // ^S
            {
               shoulderLength -= 0.02;
            }
            else if (e.isControlDown() && e.getKeyCode() == 83) // ^s
            {
               shoulderLength += 0.02;
            }
            else if (e.isControlDown() && e.isShiftDown() && e.getKeyCode() == 69) // ^E
            {
               elbowLength -= 0.02;
            }
            else if (e.isControlDown() && e.getKeyCode() == 69) // ^e
            {
               elbowLength += 0.02;
            }
            else if (e.isControlDown() && e.isShiftDown() && e.getKeyCode() == 87) // ^W
            {
               wristLength -= 0.02;
            }
            else if (e.isControlDown() && e.getKeyCode() == 87)  // ^w
            {
               wristLength += 0.02;
            }
            else if (e.isControlDown() && e.isShiftDown() && e.getKeyCode() == 70) // ^F
            {
               fingerLength -= 0.02;
            }
            else if (e.isControlDown() && e.getKeyCode() == 70) // ^f
            {
               fingerLength += 0.02;
            }
         }


         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
          //System.out.println( e );

            char c = e.getKeyChar();
            if ('h' == c)
            {
               print_help_message();
               return;
            }
            //else if ('d' == c)
            //{
            //   Pipeline.debug = ! Pipeline.debug;
            // //Clip.debug = ! Clip.debug;
            // //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
            //}
            //else if ('a' == c)
            //{
            //   Pipeline.doAntialiasing = ! Pipeline.doAntialiasing;
            //}
            else if ('x' == c)
            {
               xTranslation += 0.02;
            }
            else if ('X' == c)
            {
               xTranslation -= 0.02;
            }
            else if ('y' == c)
            {
               yTranslation += 0.02;
            }
            else if ('Y' == c)
            {
               yTranslation -= 0.02;
            }
            else if ('s' == c)
            {
               shoulderRotation += 2.0;
            }
            else if ('S' == c)
            {
               shoulderRotation -= 2.0;
            }
            else if ('e' == c)
            {
               elbowRotation1 += 2.0;
            }
            else if ('E' == c)
            {
               elbowRotation1 -= 2.0;
            }
            else if ('w' == c)
            {
               wristRotation1 += 2.0;
            }
            else if ('W' == c)
            {
               wristRotation1 -= 2.0;
            }
            else if ('f' == c)
            {
               fingerRotation1 += 2.0;
            }
            else if ('F' == c)
            {
               fingerRotation1 -= 2.0;
            }
            else if ('d' == c)
            {
               elbowRotation2 += 2.0;
            }
            else if ('D' == c)
            {
               elbowRotation2 -= 2.0;
            }
            else if ('a' == c)
            {
               wristRotation2 += 2.0;
            }
            else if ('A' == c)
            {
               wristRotation2 -= 2.0;
            }
            else if ('c' == c)
            {
               fingerRotation2 += 2.0;
            }
            else if ('C' == c)
            {
               fingerRotation2 -= 2.0;
            }

            // Push the Position away from where the camera is.
            arm_p.matrix2Identity();
            arm_p.translate(xTranslation, yTranslation, -1);

            // Set the model matrices for the sub models.
            arm.nestedMatrix     = Matrix.rotateZ(shoulderRotation)
                                  .times(Matrix.scale(shoulderLength, shoulderLength, 1));
            elbow1.nestedMatrix  = Matrix.translate(1, 0, 0)
                                  .times(Matrix.rotateZ(elbowRotation1))
                                  .times(Matrix.scale(elbowLength/shoulderLength, elbowLength/shoulderLength, 1));
            elbow2.nestedMatrix  = Matrix.translate(1, 0, 0)
                                  .times(Matrix.rotateZ(elbowRotation2))
                                  .times(Matrix.scale(elbowLength/shoulderLength, elbowLength/shoulderLength, 1));
            wrist1.nestedMatrix  = Matrix.translate(1, 0, 0)
                                  .times(Matrix.rotateZ(wristRotation1))
                                  .times(Matrix.scale(wristLength/elbowLength, wristLength/elbowLength, 1));
            wrist2.nestedMatrix  = Matrix.translate(1, 0, 0)
                                  .times(Matrix.rotateZ(wristRotation2))
                                  .times(Matrix.scale(wristLength/elbowLength, wristLength/elbowLength, 1));
            finger1.nestedMatrix = Matrix.translate(1, 0, 0)
                                  .times(Matrix.rotateZ(fingerRotation1))
                                  .times(Matrix.scale(fingerLength/wristLength, fingerLength/wristLength, 1));
            finger2.nestedMatrix = Matrix.translate(1, 0, 0)
                                  .times(Matrix.rotateZ(fingerRotation2))
                                  .times(Matrix.scale(fingerLength/wristLength, fingerLength/wristLength, 1));

            // Render again.
            FrameBuffer fb = this.fbp.getFrameBuffer();
            fb.clearFB(Color.green);
            Pipeline.render(scene, fb);
            fbp.update();
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
          //System.out.println( e );

            // Get the new size of the FrameBufferPanel.
            int w = this.fbp.getWidth();
            int h = this.fbp.getHeight();

            // Create a new FrameBuffer that fits the new window size.
            FrameBuffer fb = new FrameBuffer(w, h);
            this.fbp.setFrameBuffer(fb);
            fb.clearFB(Color.green);
            Pipeline.render(scene, fb);
            fbp.update();
            repaint();
         }
      };
   }
}
