/*

*/

import renderer.scene.*;

import java.awt.Color;

/**
   This file defines a nested model out of two copies of Model_1.
<p>
   The tree for this model is shown below.
<p>
   Remember that every model node in the tree contains a matrix,
   geometry (vertices and line-segments) and a list of nested models.
   The geometry may be empty, and the list of nested models may also
   be empty, but the matrix cannot be "empty" (if you don't give it
   a value, then it is the identity matrix, I).
<p>
<pre>{@code
               Model_2
              /   |   \
             /    |    \
       Matrix   empty   nested Models
         I               /         \
                        /           \
                       /             \
                   Model_1           Model_1
                  /    |            /    |
                 /     |           /     |
           Matrix     geom     Matrix   geom
             I                  TSR
}</pre>
*/
public class Model_2 extends Model
{
   public Model_2(Color c1, Color c2)
   {
      super();

      Model m1 = new Model_1();
      Model m2 = new Model_1();
      m1.setColor(c1);
      m2.setColor(c2);

      m2.nestedMatrix = Matrix.translate(1, -2-Math.sqrt(2), 0)
                       .times(Matrix.scale(0.5, 0.5, 1))
                       .times(Matrix.rotateZ(-45));

      this.nestedModels.add(m1);
      this.nestedModels.add(m2);
   }
}
